import numpy as np

# Para realizar la transformada de Fourier
from scipy.fft import rfft, rfftfreq
from scipy.signal.windows import get_window as wndw

# Para crear la ventana móvil (ventanas)
from numpy.lib.stride_tricks import sliding_window_view as swv

# Para graficar

def xf_calc(N, dt) -> int:
    return rfftfreq(N, dt)[: N // 2]


def superimposed(signal, fs):
    """
    Superimposed components of a signal

    Parameters
    ----------

    signal: numpy array
        Array of shape (1, n).

    fs: int
        Signal's sampling frequency.
    """
    n_first_cycle = int(fs / 60)
    N = len(signal)

    si_signal1 = [0 for i in range(n_first_cycle)]
    si_signal2 = [round(signal[i] - signal[i - 64], 9) for i in range(n_first_cycle, N)]

    return np.array(si_signal1 + si_signal2)


def fourier(window, N, dt):
    """
    Returns fft of input signal

    parameters
    ----------

    window: np.array
        Window with signal

    N: int
        Number of samples inside window

    returns
    -------

    numpy.array
        fft coefficients
    """
    xf = xf_calc(N, dt)
    yf = rfft(window)
    try:
        yf.shape[1]
        return xf, (2.0 / N) * np.abs(yf[:, 0 : N // 2])
    except IndexError:
        return xf, (2.0 / N) * np.abs(yf[0 : N // 2])


# def fourier_windows(windows, N):

#     for window in windows:
#         yf = rfft(window)
#         return (2.0 / N) * np.abs(yf[0 : N // 2])


def moving_window(data, N, step, window_name="boxcar"):
    """
    Creación de las ventanas móviles para todas las señales

    parameters
    ----------

    data: np.array
        señal en el tiempo

    N: int
        Tamaño de la ventana (muestras)

    step: int
        Tamaño de paso de las ventanas


    returns
    -------

    np.array
        Array con ventanas de tamaño N
    """
    window_func = wndw(window_name, N, fftbins=True)
    ventanas_punto_a_punto = swv(data, (N))
    return ventanas_punto_a_punto[0::step]


def iterador_max_val(windows, window_function, N, dt, signal_name):
    xf = rfftfreq(N, dt)[: N // 2]
    window_max = np.empty(len(xf))

    for window in windows:

        window = window * window_function
        fft_window = fourier(window, N)

        for freq, (value, prev_max) in enumerate(zip(fft_window, window_max)):
            if value > prev_max:
                window_max[freq] = value

        print(f"Frequency Max {signal_name}")
        for i, max_val in enumerate(window_max):
            print(f"Frequency {60*i:4}: {max_val:7.3f}")






import numpy as np


def harmonic_distortion(FFT_window, fundamental):

    fault_harmonics = [2, 3, 4]
    caps_harmonics = [5, 6, 7, 8]
    # fundamental =
    HDF = []  # Faults harmonic distortion
    HDC = []  # Caps harmonic distortion
    for harmonic in fault_harmonics:
        component = FFT_window[harmonic] / fundamental
        HDF.append(component)
    for harmonic in caps_harmonics:
        component = FFT_window[harmonic] / fundamental
        HDC.append(component)

    HDF = np.sqrt(np.sum(np.multiply(HDF, HDF)))
    HDC = np.sqrt(np.sum(np.multiply(HDC, HDC)))
    return HDF, HDC


def detector(HDF, HDC, thld=0.1):
    if (HDF > thld) or (HDC > thld):
        if HDF > HDC:
            return 1
        elif HDF < HDC:
            return 2
    return 0


def detection_iter(FFT, fundamental, return_THD=False):
    """
    Iterates through FFT moving windows and returns trip signal

    Parameters
    ----------

    FFT: numpy array
        shape (n, 32)

    fundamnetal: int (optional)
        system's fundamental frequency

    return_THD: boolean (optional)
        wether to return HDF and HDC

    returns
    -------

    TRIP: list
        list of len n

    HDF: list
        list of len n, indices of fault harmonic distortion

    HDC: list
        list of len n, indices of capacitor harmonic distortion
    """

    HDF = []
    HDC = []
    TRIP = []
    print(f"FFT shape: {FFT.shape}")
    # print(f"fundamental: {fundamental.shape}")
    for FFT_window, fundamental in zip(FFT, fundamental):
        hdf, hdc = harmonic_distortion(FFT_window, fundamental)
        TRIP.append(detector(hdf, hdc))

        if return_THD:
            HDF.append(hdf)
            HDC.append(hdc)

    if return_THD:
        return TRIP, HDF, HDC
    return TRIP



import utils_tesis.integration as itg
from utils_tesis.signalload import CSV_pandas

signal_info = {}
signal_info["window_length"] = 64
signal_info["step"] = 4
signals = CSV_pandas()
signals.relay_list()
signal_info["signals"] = signals
signal_info["signal_name"] = "I: X0023A-R1A"
t, trip = itg.img_trip(signal_info)
import matplotlib.pyplot as plt

plt.stem(t, trip)
plt.show()



from tkinter import filedialog
import tkinter as tk
import numpy as np
import pandas as pd
import csv


class CSV_pandas:
    """Class to load and extract signals from CSV into Pandas DataFrame and numpy arrays"""

    def __init__(self):
        """Constructor
        Once class instance is created, a window will be opened to
        search for CSV file.
        """
        self.csv_load()

    def csv_load(self):
        """Method to open a file exporer
        window to load CSV file from system"""
        root = tk.Tk()
        root.withdraw()
        root.attributes("-topmost", True)
        root.lift()

        self.path = filedialog.askopenfilename(
            multiple=False,
            title="cargue CSV",
            filetypes=(("archivos CSV", "*.csv"),),
        )
        print(self.path)

        self.nombre = self.path[self.path.rfind("/") + 1 :]
        self.nombre = self.nombre.replace(".csv", "")
        print(f"Se seleccionó {self.nombre}")
        root.destroy()
        self.extraer_csv()

    # métodos privados
    def extraer_csv(self):
        """Method to create pandas dataframe from CSV file"""

        # para encontrar el tipo de delimitador del archivo .csv
        with open(self.path, "r") as f:
            header = next(f)
            sniffer = csv.Sniffer()
            delimiter = sniffer.sniff(header).delimiter

        df = pd.read_csv(self.path, delimiter=delimiter)
        # Obtener un número par de muestras
        if len(df) % 2 != 0:
            df.drop(df.tail(1).index, inplace=True)

        self.df = self.__rename_columns(df)

    def __rename_columns(self, df: pd.DataFrame) -> pd.DataFrame:
        """Private method, renames columns from Dataframe into more
        readable names. i.e. it creates a new name for each column by
        merging the three original named columns returned on CSV from
        ATP

        Parameters
        ----------
        df : pd.DataFrame
            Signals dataframe

        Returns
        -------
        pd.DataFrame
            Signals dataframe with renamed columns
        """
        # Extraer los labels de las filas 1 y 2
        pattern = r"^b'([\w ]*)'"
        node_from = df.iloc[0].str.replace(pattern, r"\1", regex=True).str.strip()
        node_to = df.iloc[1].str.replace(pattern, r"\1", regex=True).str.strip()
        df = df.drop(index=0)
        df = df.drop(index=1)

        df = self.__columns_replace(df)

        # Extraer si la medición es V, I o models; borrar si es Model
        labels_list = df.columns.values.tolist()

        # convertir los labels de cada nodo a una lista
        node_from_list = node_from.values.tolist()
        node_to_list = node_to.values.tolist()

        self.labels_list = self.__new_labels(node_from_list, node_to_list, labels_list)

        df.columns = self.labels_list
        self.labels_list = self.labels_list[1:]
        df.set_index("time")
        df = df.reset_index(drop=True)
        return df

    def __columns_replace(self, df: pd.DataFrame) -> pd.DataFrame:
        """Private method, standarizes columns names, making it easier to rename them
        according to signal type they represent.

        eg. for different CSV with time column named as: "Time", "TIME" or "time", all will
        be returned as "time"


        Parameters
        ----------
        df : pd.DataFrame
            Signals dataframe

        Returns
        -------
        pd.DataFrame
            Signals dataframe with standarized columns names
        """
        patterns = [
            (" ", ""),
            (r"^(V|I)[.\w-]*", r"\1"),
            (r"^(C)[.\w-]*", r"I"),
            (r"^(?![\s\S])", r"Model"),
            (r"^\d*\.*\d+", r"Model"),
            (r"MODELS\.*[\d]*", r"Model"),
            (r"Time", r"time"),
            (r"b'", r""),
        ]
        # INFORMACIÓN DE Regex en patterns
        # Quitar los espacios blancos
        # Columnas que comienzan con V-xxx, I-xxx [SEM] o Voltage [Manual]
        # Columnas que comienzan con Current [manual]
        # Columnas que estén vacías [Manual]
        # Columnas cuyos nombres sean números [SEM]
        # Columnas que tengan MODELS en mayúscula [Manual]
        # Hacer la T de time minúscula

        # Crear un diccionario e iterar sobre este para reemplazar el str.replace(x,y)
        for pattern in patterns:
            df.columns = df.columns.str.replace(pattern[0], pattern[1], regex=True)
        return df

    # función para devolver la lista de labels correcta
    def __new_labels(self, list1: list, list2: list, labels: list) -> list:
        """Private method, merges three labels rows from each column into one row

        Parameters
        ----------
        list1 : list
            FROM node list. ROW 2 in CSV
        list2 : list
            TO node list. ROW 3 in CSV
        labels : list
            Signal type list (current, voltage or MODEL). ROW 1 in CSV

        Returns
        -------
        list
            List with correct column labels
        """

        final_list = []
        for x, y, z in zip(list1, list2, labels):
            if z == "time":
                final_list.append(z)
            elif y == "":
                final_list.append(f"{z}: {x}")
            elif z == "Model":
                final_list.append(f"{x}: {y}")
            else:
                final_list.append(f"{z}: {x}-{y}")
        return final_list

    # Método públicos
    def load_data(self, relay_name: str) -> tuple[np.ndarray, np.ndarray, dict]:
        """Loads a signal from CSV given a relay_name (signal name) as input

        Parameters
        ----------
        relay_name : str
            Name of signal to load. Use .relay_list method to list all available signals

        Returns
        -------
        tuple[np.ndarray, np.ndarray, dict]
            signal array, time array, dictionary with signal parameters
        """
        try:
            params = {}
            signals = self.df[["time", relay_name]]
            signals = signals.to_numpy().astype(float)
            if len(signals[1]) % 2 == 0:
                n_samples = len(signals)
            else:
                n_samples = len(signals) - 1

            signals = signals[:n_samples:8]
            t = signals[:, 0]
            x = signals[:, 1]
            tf = max(t)
            ti = t[0]
            fs = int((len(t) - 1) / (tf - 0))
            dt = 1 / fs
            params["tf"] = tf
            params["ti"] = ti
            params["fs"] = fs
            params["dt"] = dt
            params["n_samples"] = n_samples
            return x, t, params

        except KeyError:
            print(
                f"There is no signal '{relay_name}\n please use .relay_list() method.'"
            )
            exit()

    def relay_list(
        self, currents: bool = True, voltages: bool = False, MODELS: bool = False
    ):
        """Method, prints a list of available signals. Only lists currents by default.
        To list voltages use voltages = True.
        To list MODELS use Models = True.

        Parameters
        ----------
        currents : bool, optional
            Whether or not to print available current signals,
            by default True
        voltages : bool, optional
            Wheter or not to print available voltage signals,
            by default False
        Models : bool, optional
            Wheter or not to print available MODELS signals,
            by default False
        """
        for i in self.labels_list:
            if ("V:" in i) and voltages:
                print(i)
            elif ("I:" in i) and currents:
                print(i)
            elif MODELS:
                print(i)


# Clase para hacer pruebas, luego borrar
# no muestra ventana de selección
class CSV_pandas_path(CSV_pandas):
    """Class to load CSV with path and not by opening
    a file explorer window

    Parameters
    ----------
    CSV_pandas : parent class
        Parent class from which CSV_pandas_path inherits from
    """

    def __init__(self, filename: str):
        """Constructor

        Parameters
        ----------
        filename : str
            Path of CSV
        """

        self.csv_load(filename)

    def csv_load(self, filename: str):
        """Function to load CSV

        Parameters
        ----------
        filename : str
            Path of CSV
        """
        self.path = filename
        # self.path = "C:\\Users\\aherrada\\OneDrive - Universidad del Norte\\Uninorte\\DetectionDataBase\\septDataBaseCSV\\Caps\\NoFault02_B112.csv"
        self.extraer_csv()


def synthetic_signal(t, harmonics=[60], fundamental=60):
    """
    Function for the creation of a synthetic signal



    Returns
    -------
    numpy.array
        (3, n) three phase matrix
    """
    wt = 2 * np.pi * fundamental * t
    ang_A = float(0) * np.pi / 180
    ang_B = float(240) * np.pi / 180
    ang_C = float(120) * np.pi / 180

    A, B, C = 0, 0, 0

    for s in harmonics:
        n = s / fundamental
        A += 100 * np.sin(n * (wt + ang_A))
        B += 100 * np.sin(n * (wt + ang_B))
        C += 100 * np.sin(n * (wt + ang_C))

    return np.array((A, B, C))


class SyntheticSignal:
    """Creates a synthetic signal"""

    def __init__(
        self, samples_cycle=64, cycles=4, frequency_components=[60, 120]
    ) -> None:
        """Constructor of the signal

        Parameters
        ----------

        samples_cycle: int
            Number of samples in a period of the signal.

        cycles: int
            Number of cycles of the signal.

        frequency_components: list
            List containing frequencies (int) in the signal.
        """
        self.dt = 1 / (60 * samples_cycle)
        self.t = np.arange(0, cycles / 60, self.dt)
        self.tf = self.t[-1]
        self.fs = int(1 / self.dt)
        wt = 2 * np.pi * 60 * self.t



from utils_tesis.auxfunctions import superimposed, moving_window, fourier
from utils_tesis.detection import detection_iter
import numpy as np
from itertools import repeat


def signal_t(
    signal_info: dict, no_return: bool = False
) -> tuple[np.ndarray, np.ndarray]:
    """Returns time and signal vectors

    Parameters
    ----------
    signal_info : dict
        Dictionary with signal parameters. Must contain 'signals' and 'signal_name'
    no_return : bool, optional
        Internal variable to use when accessing from other functions from module,
        by default False

    Returns
    -------
    tuple[np.ndarray, np.ndarray]
        time vector, signal vector
    """
    # Extraer la información del diccionario
    signal_name = signal_info["signal_name"]
    signals = signal_info["signals"]
    # Cargar la señal
    signal, t, params = signals.load_data(signal_name)
    # Guardar los valores en el diccionario
    signal_info["signal"] = signal
    signal_info["t"] = t
    signal_info["params"] = params

    if no_return:
        return

    return t, signal


def si_signal_t(
    signal_info: dict, no_return: bool = False
) -> tuple[np.ndarray, np.ndarray]:
    """Returns time and superimposed signal vectors

    Parameters
    ----------
    signal_info : dict
        Dictionary with signal parameters. Must contain 'signal',
        'signals' and 'signal_name'
    no_return : bool, optional
        Internal variable to use when accessing from other functions from module,
        by default False

    Returns
    -------
    tuple[np.ndarray, np.ndarray]
        time vector, superimposed signal vector
    """
    signal = signal_info.get("signal", "")

    if len(signal) == 0:
        signal_t(signal_info, no_return=True)

    signal = signal_info["signal"]
    fs = signal_info["params"]["fs"]
    t = signal_info["t"]

    si_signal = superimposed(signal, fs)

    signal_info["si_signal"] = si_signal
    if no_return:
        return

    return t, si_signal


def windows_creator(
    signal_info: dict, no_return: bool = False
) -> tuple[np.ndarray, np.ndarray]:
    """Returns time and signal windows

    Parameters
    ----------
    signal_info : dict
        Dictionary with signal parameters. Must contain 'signal', 'window_length',
        'step', 'signals' and 'signal_name'
    no_return : bool, optional
        Internal variable to use when accessing from other functions from module,
        by default False

    Returns
    -------
    tuple[np.ndarray, np.ndarray]

        time windows, signal windows
        shape -> (m, N)
            with:

            m = ((M - N)/step) + 1

            where
            m = amount of windows
            N = windows length
            M = signal length

    """
    signal = signal_info.get("signal", "")

    if len(signal) == 0:
        signal_t(signal_info, no_return=True)

    signal = signal_info["signal"]
    t = signal_info["t"]
    window_length = signal_info["window_length"]
    step = signal_info["step"]
    signal_windows, t_windows = list(
        map(moving_window, [signal, t], repeat(window_length), repeat(step))
    )

    signal_info["signal_windows"] = signal_windows
    signal_info["t_windows"] = t_windows

    if no_return:
        return

    return t_windows, signal_windows


def si_windows_creator(
    signal_info: dict, no_return: bool = False
) -> tuple[np.ndarray, np.ndarray]:
    """Returns time and superimposed signal windows

    Parameters
    ----------
    signal_info : dict
        Dictionary with signal parameters. Must contain 'si_signal', 'window_length',
        'step', 'signals' and 'signal_name'
    no_return : bool, optional
        Internal variable to use when accessing from other functions from module,
        by default False

    Returns
    -------
    tuple[np.ndarray, np.ndarray]

        time windows, superimposed signal windows
        shape -> (m, N)
            with:

            m = ((M - N)/step) + 1

            where
            m = amount of windows
            N = windows length
            M = signal length

    """

    si_signal = signal_info.get("si_signal", "")
    if len(si_signal) == 0:
        si_signal_t(signal_info, no_return=True)

    si_signal = signal_info["si_signal"]
    t = signal_info["t"]
    window_length = signal_info["window_length"]
    step = signal_info["step"]

    si_signal_windows, t_windows = list(
        map(moving_window, [si_signal, t], repeat(window_length), repeat(step))
    )

    signal_info["si_signal_windows"] = si_signal_windows
    signal_info["t_windows"] = t_windows

    if no_return:
        return

    return t_windows, si_signal_windows


def STFT(signal_info: dict, no_return: bool = False) -> tuple[np.ndarray, np.ndarray]:
    """Returns short time Fourier transform of signal.

    Parameters
    ----------
    signal_info : dict
        Dictionary with signal parameters. Must contain 'signal_windows', 'window_length',
        'step', 'dt', 'signals' and 'signal_name'
    no_return : bool, optional
        Internal variable to use when accessing from other functions from module,
        by default False

    Returns
    -------
    tuple[np.ndarray, np.ndarray]
        Frecuency bins array, Fast Fourier Transform vector
    """

    signal_windows = signal_info.get("signal_windows", "")
    if len(signal_windows) == 0:
        windows_creator(signal_info, no_return=True)

    signal_windows = signal_info["signal_windows"]
    dt = signal_info["params"]["dt"]
    window_length = signal_info["window_length"]

    xf, fft_windows = fourier(signal_windows, window_length, dt)

    signal_info["xf"] = xf
    signal_info["fft_windows"] = fft_windows
    signal_info["fft_windows_fundamental"] = fft_windows[:, 1]

    if no_return:
        return

    number_of_windows = len(fft_windows)

    return np.tile(xf, (number_of_windows, 1)), fft_windows


def si_STFT(
    signal_info: dict, no_return: bool = False
) -> tuple[np.ndarray, np.ndarray]:
    """Returns short time Fourier transform of signal's superimposed components.

    Parameters
    ----------
    signal_info : dict
        Dictionary with signal parameters. Must contain 'si_signal_windows', 'window_length',
        'step', 'dt', 'signals' and 'signal_name'
    no_return : bool, optional
        Internal variable to use when accessing from other functions from module,
        by default False

    Returns
    -------
    tuple[np.ndarray, np.ndarray]
        Frecuency bins array, si signal Fast Fourier Transform vector
    """

    si_signal_windows = signal_info.get("si_signal_windows", "")
    if len(si_signal_windows) == 0:
        si_windows_creator(signal_info, no_return=True)

    si_signal_windows = signal_info["si_signal_windows"]
    dt = signal_info["params"]["dt"]
    window_length = signal_info["window_length"]

    xf, si_fft_windows = fourier(si_signal_windows, window_length, dt)

    signal_info["xf"] = xf
    signal_info["si_fft_windows"] = si_fft_windows
    signal_info["si_fft_windows_fundamental"] = si_fft_windows[:, 1]

    if no_return:
        return

    number_of_windows = len(si_fft_windows)

    return np.tile(xf, (number_of_windows, 1)), si_fft_windows


def trip(signal_info: dict, no_return: bool = False) -> list:
    """Returns trip signal for signal

    Parameters
    ----------
    signal_info : dict
        Dictionary with signal parameters. Must contain:
        - 'signal_windows'
        - 'window_length',
        - 'step'
        - 'dt'
        - 'signals'
        - 'signal_name'
    no_return : bool, optional
        Internal variable to use when accessing from other functions from module,
        by default False

    Returns
    -------
    list
        List containing trip values
    """

    fft_windows = signal_info.get("fft_windows", "")
    if len(fft_windows) == 0:
        STFT(signal_info, no_return=True)

    fft_windows = signal_info["fft_windows"]
    fundamental = signal_info["fft_windows_fundamental"]
    trip_windows = detection_iter(fft_windows, fundamental)

    signal_info["trip_windows"] = trip_windows

    if no_return:
        return

    return trip_windows


def si_trip(signal_info: dict, no_return: bool = False) -> list:
    """Returns trip signal for signal

    Parameters
    ----------
    signal_info : dict
        Dictionary with signal parameters. Must contain:
        - 'si_signal_windows'
        - 'fft_windows_fundamental'
        - 'window_length'
        - 'step'
        - 'dt'
        - 'signals'
        - 'signal_name'
    no_return : bool, optional
        Internal variable to use when accessing from other functions from module,
        by default False

    Returns
    -------
    list
        List containing trip values of superimposed signal
    """
    si_fft_windows = signal_info.get("si_fft_windows", "")
    fundamental = signal_info.get("fft_windows_fundamental", "")
    if len(si_fft_windows) == 0:
        si_STFT(signal_info, no_return=True)
    if len(fundamental) == 0:
        STFT(signal_info, no_return=True)

    si_fft_windows = signal_info["si_fft_windows"]
    fundamental = signal_info["fft_windows_fundamental"]
    si_trip_windows = detection_iter(si_fft_windows, fundamental)

    signal_info["si_trip_windows"] = si_trip_windows

    if no_return:
        return

    return si_trip_windows


def img_trip(signal_info: dict) -> tuple[np.ndarray, np.ndarray]:
    """Returns time vector with trip signal

    Parameters
    ----------
    signal_info : dict
        Dictionary with signal parameters. Must contain:
        - 'signal_windows'
        - 'trip windows
        - 'fft_windows_fundamental'
        - 'window_length'
        - 'step'
        - 'dt'
        - 'signals'
        - 'signal_name'

    Returns
    -------
    tuple[np.ndarray, np.ndarray]
        time vector, trip signal
    """
    trip_windows = signal_info.get("trip_windows", "")
    if len(trip_windows) == 0:
        trip(signal_info, no_return=True)

    trip_windows = signal_info["trip_windows"]
    t_windows = signal_info["t_windows"]
    t_window = np.insert(t_windows[:, -1], 0, 0)
    trip_windows = np.insert(trip_windows, 0, 0)
    return t_window, trip_windows


def img_si_trip(signal_info: dict) -> tuple[np.ndarray, np.ndarray]:
    """Returns time vector with trip signal

    Parameters
    ----------
    signal_info : dict
        Dictionary with signal parameters. Must contain:
        - 'si_signal_windows'
        - 'trip windows'
        - 't_windows'
        - 'fft_windows_fundamental'
        - 'window_length'
        - 'step'
        - 'dt'
        - 'signals'
        - 'signal_name'

    Returns
    -------
    tuple[np.ndarray, np.ndarray]
        time vector, trip signal for superimposed components
    """
    trip_windows = signal_info.get("si_trip_windows", "")
    if len(trip_windows) == 0:
        si_trip(signal_info, no_return=True)

    trip_windows = signal_info["si_trip_windows"]
    t_windows = signal_info["t_windows"]
    t_window = np.insert(t_windows[:, -1], 0, 0)
    trip_windows = np.insert(trip_windows, 0, 0)
    return t_window, trip_windows



from utils_tesis.auxfunctions import superimposed, moving_window, fourier
from utils_tesis.detection import detection_iter
import numpy as np
from itertools import repeat



def img_signal(request_information, no_return=False):
    signal_name = request_information["signal_name"]
    signals = request_information["signals"]
    signal, t, params = signals.load_data(signal_name)
    request_information["signal"] = signal
    request_information["t"] = t
    request_information["params"] = params
    request_information["plots"]["img_signal"] = True
    if no_return:
        print("no return img_signal")
        return

    return t.tolist(), signal.tolist(), "linear", "img"


def img_si_signal(request_information, no_return=False):
    signal = request_information.get("signal", "")

    if len(signal) == 0:
        print("No existe signal")

        img_signal(request_information, no_return=True)

    signal = request_information["signal"]
    fs = request_information["params"]["fs"]
    t = request_information["t"]

    si_signal = superimposed(signal, fs)

    request_information["si_signal"] = si_signal
    request_information["plots"]["img_si_signal"] = True
    if no_return:
        print("no return img_si_signal")
        return

    return t.tolist(), si_signal.tolist(), "linear", "img"


def anim_signal(request_information, no_return=False):
    signal = request_information.get("signal", "")

    if len(signal) == 0:
        print("No existe signal")
        img_signal(request_information, no_return=True)

    signal = request_information["signal"]
    t = request_information["t"]
    window_length = request_information["window_length"]
    step = request_information["step"]
    signal_windows, t_windows = list(
        map(moving_window, [signal, t], repeat(window_length), repeat(step))
    )

    request_information["signal_windows"] = signal_windows
    request_information["t_windows"] = t_windows
    request_information["plots"]["anim_signal"] = True
    max_in_windows = signal_windows.max()
    min_in_windows = signal_windows.min()
    max_min = [max_in_windows, min_in_windows]
    if no_return:
        print("no return anim_signal")
        return

    return t_windows.tolist(), signal_windows.tolist(), max_min, "anim"


def anim_si_signal(request_information, no_return=False):
    si_signal = request_information.get("si_signal", "")
    if len(si_signal) == 0:
        print("No existe si_signal")
        img_si_signal(request_information, no_return=True)

    si_signal = request_information["si_signal"]
    t = request_information["t"]
    window_length = request_information["window_length"]
    step = request_information["step"]
    si_signal_windows, t_windows = list(
        map(moving_window, [si_signal, t], repeat(window_length), repeat(step))
    )

    request_information["si_signal_windows"] = si_signal_windows
    request_information["t_windows"] = t_windows
    request_information["plots"]["anim_si_signal"] = True
    max_in_windows = si_signal_windows.max()
    min_in_windows = si_signal_windows.min()
    max_min = [max_in_windows, min_in_windows]

    if no_return:
        print("no return anim_si_signal")
        return

    return t_windows.tolist(), si_signal_windows.tolist(), max_min, "anim"


def anim_fft(request_information, no_return=False):
    signal_windows = request_information.get("signal_windows", "")
    if len(signal_windows) == 0:
        print("No existe signal_windows")
        anim_signal(request_information, no_return=True)

    signal_windows = request_information["signal_windows"]
    dt = request_information["params"]["dt"]
    window_length = request_information["window_length"]

    xf, fft_windows = fourier(signal_windows, window_length, dt)

    request_information["xf"] = xf
    request_information["fft_windows"] = fft_windows
    request_information["fft_windows_fundamental"] = fft_windows[:, 1]
    request_information["plots"]["anim_fft"] = True
    max_in_windows = fft_windows.max()
    min_in_windows = fft_windows.min()
    max_min = [max_in_windows, min_in_windows]

    if no_return:
        print("no return anim_fft")
        return

    number_of_windows = len(fft_windows)

    return (
        np.tile(xf, (number_of_windows, 1)).tolist(),
        fft_windows.tolist(),
        max_min,
        "STFT",
    )


def anim_si_fft(request_information, no_return=False):
    si_signal_windows = request_information.get("si_signal_windows", "")
    if len(si_signal_windows) == 0:
        print("No existe si_signal_windows")
        anim_si_signal(request_information, no_return=True)

    si_signal_windows = request_information["si_signal_windows"]
    dt = request_information["params"]["dt"]
    window_length = request_information["window_length"]

    xf, si_fft_windows = fourier(si_signal_windows, window_length, dt)

    request_information["xf"] = xf
    request_information["si_fft_windows"] = si_fft_windows
    request_information["si_fft_windows_fundamental"] = si_fft_windows[:, 1]
    request_information["plots"]["anim_si_fft"] = True
    max_in_windows = si_fft_windows.max()
    min_in_windows = si_fft_windows.min()
    max_min = [max_in_windows, min_in_windows]

    if no_return:
        print("no return anim_fft")
        return

    number_of_windows = len(si_fft_windows)

    return (
        np.tile(xf, (number_of_windows, 1)).tolist(),
        si_fft_windows.tolist(),
        max_min,
        "STFT",
    )


def anim_trip(request_information, no_return=False):
    fft_windows = request_information.get("fft_windows", "")
    if len(fft_windows) == 0:
        print("No existe la señal")
        anim_fft(request_information, no_return=True)

    fft_windows = request_information["fft_windows"]
    fundamental = request_information["fft_windows_fundamental"]
    trip_windows = detection_iter(fft_windows, fundamental)

    request_information["trip_windows"] = trip_windows
    request_information["plots"]["anim_trip"] = True
    max_in_windows = max(trip_windows)
    min_in_windows = min(trip_windows)
    max_min = [max_in_windows, min_in_windows]

    if no_return:
        print("no return anim_trip")
        return

    return trip_windows, max_min, "anim"


def anim_si_trip(request_information, no_return=False):
    si_fft_windows = request_information.get("si_fft_windows", "")
    fundamental = request_information.get("fft_windows_fundamental", "")
    if len(si_fft_windows) == 0:
        print("No existe la señal")
        anim_si_fft(request_information, no_return=True)
    if len(fundamental) == 0:
        print("No fundamental")
        anim_fft(request_information, no_return=True)

    si_fft_windows = request_information["si_fft_windows"]
    fundamental = request_information["fft_windows_fundamental"]
    si_trip_windows = detection_iter(si_fft_windows, fundamental)

    request_information["si_trip_windows"] = si_trip_windows
    request_information["plots"]["anim_si_trip"] = True

    if no_return:
        print("no return anim_trip")
        return

    return si_trip_windows, "", "anim"


# def img_trip(request_information, no_return=False):
#     signals = request_information["signals"]
#     signal_name = request_information["signal_name"]
#     (signal_window, signal_si_window, t_window), (
#         signal_fft,
#         signal_si_fft,
#         xf,
#     ) = windows_creator(
#         64,
#         signals=signals,
#         signal_name=signal_name,
#         windows_fourier=True,
#     )
#     signal_fundamental = signal_fft[:, 1]
#     si_fundamental = signal_si_fft[:, 1]
#     trip = detection_iter(signal_fft, signal_fundamental)
#     t_window = np.insert(t_window[:, -1], 0, 0)
#     trip = np.insert(trip, 0, 0)
#     return t_window.tolist(), trip.tolist(), "hv", "img"


def img_trip(request_information, no_return=False):
    trip_windows = request_information.get("trip_windows", "")
    if len(trip_windows) == 0:
        print("No existe la señal")
        anim_trip(request_information, no_return=True)

    trip_windows = request_information["trip_windows"]
    t_windows = request_information["t_windows"]
    t_window = np.insert(t_windows[:, -1], 0, 0)
    trip_windows = np.insert(trip_windows, 0, 0)
    return t_window.tolist(), trip_windows.tolist(), "hv", "trip"


def img_si_trip(request_information, no_return=False):
    trip_windows = request_information.get("si_trip_windows", "")
    if len(trip_windows) == 0:
        print("No existe la señal")
        anim_si_trip(request_information, no_return=True)

    trip_windows = request_information["si_trip_windows"]
    t_windows = request_information["t_windows"]
    t_window = np.insert(t_windows[:, -1], 0, 0)
    trip_windows = np.insert(trip_windows, 0, 0)
    return t_window.tolist(), trip_windows.tolist(), "hv", "trip"
